# frozen_string_literal: true

# Ramverk is a web application framework written in Ruby.
module Ramverk
  require_relative "ramverk/version"
  require_relative "ramverk/configuration"
  require_relative "ramverk/resolver"

  # Get the current environment status.
  #
  # @return [Symbol]
  def self.env
    (ENV["APP_ENV"] || ENV["RACK_ENV"] || :development).to_sym
  end

  # Check if the given environment match the current.
  #
  # @overload env?(environment, ...)
  #   @param environment [Symbol]
  #   @param ... [Symbol]
  #
  # @return [Boolean]
  def self.env?(*environment)
    environment.include?(env)
  end

  # Project configuration.
  #
  # @return [Ramverk::Configuration]
  def self.configuration
    @configuration
  end

  # Configure project within a block.
  #
  # @yield [config] Configuration.
  # @yieldparam config [Ramverk::Configuration]
  #
  # @example
  #   Ramverk.configure do |config|
  #   end
  def self.configure
    yield configuration
  end

  # Boot the project.
  #
  # @return [true]
  #
  # @raise [RuntimeError] If project already been booted.
  #
  # @example
  #   Ramverk.boot
  def self.boot
    raise "project has already been booted" if @booted

    configuration.boot
    @booted = true
  end

  # Rack compatible endpoint.
  #
  # @return [#call]
  def self.rack
    boot unless @booted

    builder = Rack::Builder.new

    configuration.middleware.stack.each do |(mw, args, block)|
      builder.use mw, *args, &block
    end

    builder.run Resolver.new(configuration.apps).freeze
    builder.freeze
  end

  # Check if the program is running via Rake.
  #
  # @return [Boolean]
  def self.rake?
    File.basename($PROGRAM_NAME) == "rake"
  end

  # @private
  def self.reset!
    @booted = false
    @configuration = Configuration.new
  end

  reset!
end
