# frozen_string_literal: true

require "json"
require "logger"
require "pathname"
require "zeitwerk"
require "mustermann"

module Ramverk
  # Project configuration.
  class Configuration
    require_relative "configuration/dynamic_groups"
    require_relative "configuration/middleware"

    include DynamicGroups

    # Project root path.
    #
    # @return [Pathname]
    attr_reader :root

    # Middleware manager.
    #
    # @return [Ramverk::Middleware]
    #
    # @see Ramverk::Middleware
    attr_reader :middleware

    # Project logger.
    #
    # @return [Logger]
    attr_accessor :logger

    # Logger severity level.
    #
    # @return [Symbol]
    attr_accessor :logger_level

    # Logger formatter.
    #
    # @return [Proc]
    attr_accessor :logger_formatter

    # Params to be [FILTERED] out from logs.
    #
    # @return [Array<String>]
    attr_accessor :logger_filter_params

    # Autoloader object.
    #
    # @return [Zeitwerk::Loader]
    attr_reader :autoload_loader

    # Paths to (re)load constants from. All paths should be relative to project
    # root path.
    #
    # @return [Array]
    attr_accessor :autoload_paths

    # Eager loads all files in the root of the #autoload_paths directories,
    # recursively.
    #
    # Disabled by default in `development` and `test` environments.
    #
    # @return [Boolean]
    attr_accessor :autoload_eager_load

    # Reload constants automatically. This is only enabled in `development` by
    # default.
    #
    # Enabled by default in `development` environment.
    #
    # @return [Boolean]
    attr_accessor :autoload_reload

    # Public directory folder.
    #
    # @return [String]
    attr_accessor :public_directory

    # Serve static files.
    #
    # @return [Boolean]
    attr_accessor :serve_static_files

    # Mounted applications.
    #
    # @return [Hash]
    attr_reader :apps

    # @private
    # rubocop:disable Metrics/AbcSize
    def initialize(env: Ramverk.env)
      @env = env
      @root = Pathname.new(Dir.pwd)
      @apps = {}
      @public_directory = "public"
      @serve_static_files = env == :development

      # Middleware
      @middleware = Middleware.new

      # Logging
      @logger = ::Logger.new(env == :test ? IO::NULL : $stdout)
      @logger_level = env == :production ? :info : :debug
      @logger_formatter = LOGGER_DEFAULT_FORMATTER
      @logger_filter_params = %w[password password_confirmation password_hash]

      # Autoloading
      @autoload_loader = Zeitwerk::Loader.new
      @autoload_paths = %w[]
      @autoload_eager_load = env != :development
      @autoload_reload = env == :development
    end

    # Mount a Rack compatible at the given path prefix.
    #
    # @param app [#call]
    #   Application to mount.
    # @param at [String]
    #   Path prefix to match.
    # @param as [Symbol]
    #   App identifier.
    # @param host [Regexp]
    #   Match the given host pattern.
    def mount(app, at:, as:, host: nil)
      @apps[as] = [app, Mustermann.new(at), host].freeze
    end

    # Yield the block if the given environment matches the current.
    #
    # @param env [Symbol]
    # @yieldparam config [Ramverk::Configuration]
    #
    # @example
    #   Ramverk.configure do |config|
    #     config.environment :development do
    #     end
    #   end
    def environment(env)
      yield self if env == @env
    end

    # Set project root path.
    #
    # @param value [String, Pathname]
    #
    # @raise [Errno::ENOENT]
    def root=(value)
      @root = Pathname.new(value).realpath
    end

    # @private
    def boot
      boot_logger
      boot_autoload
      boot_apps
      boot_static_file_server

      freeze
    end

    # @private
    def freeze
      apps.freeze
      middleware.freeze

      super
    end

    private

    # @private
    def boot_logger
      return unless logger

      logger.level = logger_level
      logger.formatter = logger_formatter
      logger.freeze

      require_relative "middleware/request_logger"
      middleware.prepend Ramverk::Middleware::RequestLogger,
                         logger,
                         logger_filter_params
    end

    # @private
    # rubocop:disable Metrics/AbcSize
    def boot_autoload
      return if !autoload_paths || autoload_paths.empty?

      autoload_paths.each { |path| autoload_loader.push_dir(root.join(path)) }

      if autoload_reload
        autoload_loader.enable_reloading

        require_relative "middleware/reloader"
        middleware.prepend Ramverk::Middleware::Reloader, autoload_loader
      end

      autoload_loader.setup
      Zeitwerk::Loader.eager_load_all if autoload_eager_load
    end
    # rubocop:enable Metrics/AbcSize

    # @private
    def boot_apps
      apps.each_value do |(app, _pattern, _host)|
        app.boot if app.respond_to?(:boot)
      end
    end

    # @private
    def boot_static_file_server
      return unless serve_static_files

      require_relative "middleware/static"
      middleware.prepend Ramverk::Middleware::Static, public_directory
    end

    # @private
    LOGGER_DEFAULT_FORMATTER = lambda do |severity, time, _progname, message|
      message = { severity: severity, time: time, message: message }
      %(#{message.to_json}\n)
    end
    private_constant :LOGGER_DEFAULT_FORMATTER
  end
end
