# frozen_string_literal: true

module Ramverk
  # Conveniences for inflecting and working with string manipulation.
  module String
    module_function

    # Return a downcased and underscore separated version of the string.
    #
    # @param string [String]
    #   String to be transformed.
    #
    # @return [String]
    #   The transformed string.
    #
    # @example
    #   string = "RamverkString"
    #   Ramverk::String.underscore(string) # => 'ramverk_string'
    def underscore(string)
      string = +string.to_s
      string.gsub!("::", "/")
      string.gsub!(/([A-Z\d]+)([A-Z][a-z])/, '\1_\2')
      string.gsub!(/([a-z\d])([A-Z])/, '\1_\2')
      string.gsub!(/[[:space:]]|\-/, '\1_\2')
      string.downcase!
      string
    end

    # Return a CamelCase version of the string.
    #
    # @param string [String]
    #   String to be transformed.
    #
    # @return [String]
    #   The transformed string
    #
    # @example
    #   string = "ramverk_string"
    #   Ramverk::String.classify(string) # => 'RamverkString'
    def classify(string)
      words = underscore(string).split(%r{_|::|\/|\-}).map!(&:capitalize)
      delimiters = underscore(string).scan(%r{_|::|\/|\-})
      delimiters.map! { |delimiter| delimiter == "_" ? "" : "::" }
      words.zip(delimiters).join
    end
  end
end
